/*!\file:  CreateSingleNodeToElementConnectivity.cpp
 * \brief: create connectivity table
 */ 

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif

#include "../../shared/shared.h"
#include "../../io/io.h"
#include "../../include/include.h"
#include "./ModelProcessorx.h"

void CreateSingleNodeToElementConnectivity(IoModel* iomodel){

	/*Intermediary*/
	int i,j;
	int vertexid;
	int elementswidth;

	/*output*/
	int* connectivity=NULL;

	/*Some checks if debugging*/
	_assert_(iomodel->numberofvertices);
	_assert_(iomodel->numberofelements);
	_assert_(iomodel->my_elements);
	_assert_(iomodel->elements);

	/*Allocate ouput*/
	connectivity=(int*)xcalloc(iomodel->numberofvertices,sizeof(int));

	/*Get element width (3 or 6)*/
	if (iomodel->dim==2){
		elementswidth=3;
	}
	else{
		elementswidth=6;
	}

	/*Create connectivity table*/
	for (i=0;i<iomodel->numberofelements;i++){
		/*!! in parallel we do not want the vertex to be connected to an element that is not in its partition!!*/
		if(iomodel->my_elements[i]){
			for (j=0;j<elementswidth;j++){
				vertexid=(int)iomodel->elements[elementswidth*i+j];
				_assert_(vertexid>0 && vertexid-1<iomodel->numberofvertices);
				connectivity[vertexid-1]=i+1;
			}
		}
	}

	/*Assign to iomodel*/
	iomodel->singlenodetoelementconnectivity=connectivity;

	/*Do not free connectivity!!!*/
}
